import argparse

def parser():
    parser = argparse.ArgumentParser(
        prog = 'PEAR',
        description='PEAR-EBI | \
        Phylogeny Embedding and Approximate Representation \n \
        Calculates RF distances between large set of trees',
        epilog = 'Author: Andrea Rubbi - Goldman Group')
    parser.add_argument(type=str, dest='input', metavar = 'input',
                    help='input file : tree set in newic format')
    parser.add_argument('-o', type=str, dest='output', metavar = 'output',
                    help='output file : storage of distance matrix', required= False)
    parser.add_argument('-v', dest='verbose', action= 'store_true',
                    help='print info while running', required= False)
    parser.add_argument('--i', dest='interactive_mode',
                    help='run the program in interactive mode', 
                    required= False, action= 'store_true')
    parser.add_argument('-d', type=str, dest='distance_matrix', metavar = 'distance_matrix',
                    help='distance matrix : file of the distance matrix', required= False)
    parser.add_argument('-m', type=str, dest='metadata', metavar = 'metadata',
                    help='metadata : csv file with metadata for each tree', required= False)
    parser.add_argument('-hashrf', '--h', dest='hashrf', action= 'store_true',
                    help='calculates tree distances using hashrf', required= False)
    parser.add_argument('-hashrf_w', '--hw', dest='hashrf_weighted', action= 'store_true',
                    help='calculates tree distances using weighted hashrf', required= False)
    parser.add_argument('-daysRF', '--drf', dest='days_RF', action= 'store_true',
                    help='calculates tree distances using an implementation of Day\'s algorithm', required= False)
    parser.add_argument('-pca', dest='pca', type= int,
                    help='embedding using PCoA: select 2 or 3 principal components', required= False)
    parser.add_argument('-tsne', dest='tsne', type= int,
                    help='embedding using t-SNE: select 2 or 3 final dimensions', required= False)
    parser.add_argument('-plot', dest='plot', type= int,
                    help='plot embedding in 2 or 3 dimensions', required= False)
    parser.add_argument('-subset', '--s', dest='subset', type= int,
                    help='extract subset of collection', required= False)
    
    return parser.parse_args()


import sys
try: from .maple_RF import *
except: from maple_RF import *

if __name__ == "__main__":
    pckl = open('Trees.pckl', 'rb')
    trees = pickle.load(pckl)
    pckl.close()
    
    i = int(sys.argv[1])
    inputTree = trees[i:i+1]
    inputRFtrees = trees[:]
    
    tree1 = readNewick(inputTree)[0]
    tree1_prep = prepareTreeComparison(tree1, rooted=False)
    otherTrees = readNewick(inputRFtrees)
    
    
    RF_distances = list()
    for tree in otherTrees:
        res = RobinsonFouldsWithDay1985(tree, tree1_prep, rooted=False)
        RF_distances.append(res[0])
    print('\n', RF_distances, '\n')


import subprocess
import numpy as np
import pandas as pd
import os

# Set the value Display variable
os.environ.setdefault('DISPLAY', ':0.0')

# Simple function to perform bash operations 
def bash_command(cmd): 
    subprocess.run(['/bin/bash', '-c', cmd], stderr=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
    return 0

# ──────────────────────────────── RUNNING HASHRF RETURNING CLEANED OUTPUT ─────
#? From: "A RANDOMIZED ALGORITHM FOR COMPARING SETS OF PHYLOGENETIC TREES" 
#? by Seung-Jin Sul & Tiffani L. Williams
#? https://code.google.com/archive/p/hashrf/
# ──────────────────────────────────────────────────────────────────────────────

def hashrf(file, n_trees, output_file):
    """compute unweighted Robison Foulds distances

    Args:
        file (str): name of input file with phylogenetic trees in newick format
        n_trees (int): number of trees in file
        output_file (str): name of output file that will contain the distance matrix

    Returns:
       distance_matrix (pandas.DataFrame): computed distance matrix
    """
    try: path = os.path.dirname(os.path.abspath(__file__))
    except: path = '.'
    cmd = '{path}/HashRF/hashrf {file} {n_trees} -p matrix -o {output} 1> /dev/null 2> ./hashrf_err.txt'.format(path = path,
        file= file, n_trees= n_trees, output= output_file, verbose = 1)
    
    
    # Runs command
    bash_command(cmd) 

    # Reads output file and creates numpy array
    # which is used to create a pandas dataframe
    try:
        with open(output_file, 'r') as out:
                distance_matrix = pd.read_csv(out, index_col=None, header=None, sep=' ')
                out.close()
        
        distance_matrix.drop(distance_matrix.columns[-1], axis=1, inplace=True)
        distance_matrix.to_csv(output_file)
        return distance_matrix
    except: print("hashrf failed! check errors in hashrf_err.txt")
    

# HashRF calculating weighted RF distances
def hashrf_weighted(file, n_trees, output_file):
    """compute weighted Robison Foulds distances

    Args:
        file (str): name of input file with phylogenetic trees in newick format
        n_trees (int): number of trees in file
        output_file (str): name of output file that will contain the distance matrix

    Returns:
       distance_matrix (pandas.DataFrame): computed distance matrix
    """
    try: path = (os.path.dirname(__file__))
    except: path = '.'
    cmd = '{path}/HashRF/hashrf {file} {n_trees} -p matrix -o {output} -w 1> /dev/null 2> ./hashrf_err.txt'.format(path = path,
        file= file, n_trees= n_trees, output= output_file, verbose = 1)
    
    # Runs command
    bash_command(cmd) 
    bash_command(f"tr -s \" \" < {output_file} | sed 's/^[ \t]*//' > ./tmp_file && cat ./tmp_file > {output_file} && rm ./tmp_file") # removes double spaces in file and spaces at the beginning of lines
    
    # Reads output file and creates numpy array
    # which is rused to create a pandas dataframe
    try:
        with open(output_file, 'r') as out:
                distance_matrix = pd.read_csv(out, index_col=None, header=None, sep=' ')
                out.close()
        
        distance_matrix.drop(distance_matrix.columns[-1], axis=1, inplace=True)
        distance_matrix.to_csv(output_file)
        return distance_matrix
    except: print("hashrf failed! check errors in hashrf_err.txt")



        
        
        
        
        
        
        
        
    
        













__author__ = "Andrea Rubbi"
# ──────────────────────────────────────────────────────────────────────────────
import numpy as np
import pandas as pd
from sklearn.manifold import TSNE
# ──────────────────────────────────────────────────────────────────────────────
# ─── t-SNE ND ─────────────────────────────────────────────────────────────────
def tsne(distance_matrix, n_dimensions, metadata=None):
    """embed distance_matrix in n_components with t-Stochastic Neighbor Embedding

    Args:
        distance_matrix (pandas.DataFrame): distance_matrix
        n_dimensions (int): number of desired dimensions
        metadata (pandas.DataFrame, optional): metadata of elements. Defaults to None.

    Returns:
        Distances_embedded_ND (numpy.array): distances embedded in n_dimensions 
    """    
    method = 'barnes_hut'
    if n_dimensions > 3: method = 'exact'
    
    Distances_embedded_ND = TSNE(n_components=n_dimensions, method=method,
            init='random', learning_rate=200.0, perplexity=3).fit_transform(distance_matrix.values.astype(np.float32))
    pd.DataFrame(Distances_embedded_ND).to_csv('./t-SNE_Embedding.csv')

    return Distances_embedded_ND



from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='pear_ebi',
    version='0.1.1.4',
    license='MIT License',
    description='Embeds phylogenetic tree distances and produce representations',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Andrea Rubbi',
    author_email='andrea.rubbi.98@gmail.com',
    url='https://github.com/AndreaRubbi/TreeEmbedding',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'numpy',
        'pandas',
        'matplotlib',
        'scikit-learn',
        'plotly',
        'rich',
    ],
)